/*
   Linked List functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

struct ll_list *ll_head;
struct ll_list *ll_tail;
struct ll_list *ll_current;
struct ll_list *ll_browse_curr;
struct ll_list *ll_browse_top;
int ll_qty;
int ll_browse_pos;             /* Bar position */

/*
 * Initialization routine for the linked list
 */

void ll_init( void)
{
 ll_head        = NULL;
 ll_tail        = NULL;
 ll_current     = NULL;
 ll_qty         = 0;
}


/*
 * Allocate a new node
 */

struct ll_list *ll_alloc( char *filename)
{
 struct ll_list *p;

 p = ( struct ll_list *) lin_calloc( sizeof( struct ll_list) + 1);

 p->filename = ( char *) lin_calloc( strlen( filename) + 1);

 if( filename != NULL)
     strcpy( p->filename, filename);

 p->next    = NULL;
 p->prev    = NULL;

 return p;
}


/*
 * Add a new node
 */

void ll_add( char *filename, int stages)
{
 struct ll_list *p;

 p = ll_alloc( filename);

 p->stages = stages;

 if( ll_head == NULL) {
     ll_head = p;
     ll_tail = p;
 } else {
     p->prev = ll_tail;
     ll_tail->next = p;
     ll_tail = p;
 }

 ll_qty++;
}


/*
 * Free the nodes
 */

void ll_free( void)
{
 struct ll_list *p, *v;

 p = ll_head;

 while( p != NULL) {
        v = p->next;
        if( p->filename != NULL)
            free( p->filename);
        if( p != NULL)
            free( p);
        p = v;
 }

 ll_init();
}

void ll_show( void)
{
 struct ll_list *p = ll_browse_top;
 char buf[ 256];
 int y = 14;

 while( p != NULL && y < 184) {
        sprintf( buf, " %-28.28s %8d ", p->filename, p->stages);

        if( p == ll_browse_curr)
            gl_setfontcolors( LIN_WHITE, LIN_BLACK);
        else
            gl_setfontcolors( LIN_BLACK, LIN_WHITE);

        gl_write( 2, y, buf);

        y += 10;
        p = p->next;
 } /* end while( ...) */

 /* Back to font color defaults */
 gl_setfontcolors( LIN_BLACK, LIN_WHITE);

 if( y < 184)
     gl_fillbox( 0, y, 319, 184 - y, LIN_BLACK);

 if( p != NULL) {
     gl_write(   0, 190, "\x19");
     gl_write( 309, 190, "\x19");
 } else {
     gl_write(   0, 190, " ");
     gl_write( 309, 190, " ");
 }

 if( ll_browse_top->prev != NULL) {
     gl_write(  10, 190, "\x18");
     gl_write( 299, 190, "\x18");
 } else {
     gl_write(  10, 190, " ");
     gl_write( 299, 190, " ");
 }
}

int ll_browse_next( int nitems)
{
 struct ll_list *p = ll_browse_top;
 int pos = 0;

 while( p != NULL && pos++ < nitems)
        p = p->next;

 if( p == NULL)
     return -1;

 ll_browse_top = p;

 return 1;
}

int ll_browse_back( int nitems)
{
 struct ll_list *p = ll_browse_top;
 int pos = 0;

 while( p != NULL && pos++ < nitems)
        p = p->prev;

 if( p == NULL)
     return -1;

 ll_browse_top = p;

 return 1;
}

